use prost::DecodeError;
use prost_reflect::ReflectMessage;
use crate::client::*;
use crate::types::AnyExt;

pub use crate::client::watch_event::Type;

#[derive(Default, Debug)]
pub struct Event<T: prost::Message + ReflectMessage + Default> {
    pub typ: Type,
    pub old: Option<T>,
    pub new: Option<T>,
}

impl<T: prost::Message + ReflectMessage + Default> Event<T> {
    pub(crate) fn from_any(e: WatchEvent) -> Result<Event<T>, DecodeError> {
        let mut event = Event::<T>::default();
        if let Some(old) = e.old {
            event.old = match T::from_any(&old) {
                Ok(old) => Some(old),
                _ => return Err(DecodeError::new("old"))
            }
        }
        if let Some(new) = e.new {
            event.new = match T::from_any(&new) {
                Ok(new) => Some(new),
                _ => return Err(DecodeError::new("new"))
            }
        }
        event.typ = match e.r#type {
            0 => Type::Unknown,
            1 => Type::Enter,
            2 => Type::Leave,
            3 => Type::Update,
            _ => return Err(DecodeError::new("type"))
        };
        Ok(event)
    }
}
