use prost_types::{Duration, FieldMask};
use crate::protofilters::{Expression, Paging, Where};

#[derive(Default, Debug)]
pub struct GetOptions {
    pub(crate) filter: Option<Expression>,
    pub(crate) paging: Option<Paging>,
    pub(crate) field_mask: Option<FieldMask>,
}

impl GetOptions {
    pub fn with_paging(&self, paging: Paging) -> Self {
        GetOptions{paging: Some(paging), filter: self.filter.clone(), field_mask: self.field_mask.clone()}
    }
    pub fn with_filter(&self, filter: Where) -> Self {
        GetOptions{filter: Some(filter.into()), paging: self.paging.clone(), field_mask: self.field_mask.clone()}
    }
    pub fn with_field_mask(&self, field_mask: FieldMask) -> Self {
        GetOptions{field_mask: Some(field_mask), filter: self.filter.clone(), paging: self.paging.clone()}
    }
}

#[derive(Default, Debug)]
pub struct SetOptions {
    pub(crate) ttl: Option<Duration>,
    pub(crate) field_mask: Option<FieldMask>,
}

impl SetOptions {
    pub fn with_ttl(&self, ttl: Duration) -> Self {
        SetOptions{ttl: Some(ttl), field_mask: self.field_mask.clone()}
    }
    pub fn with_field_mask(&self, field_mask: FieldMask) -> Self {
        SetOptions{field_mask: Some(field_mask), ttl: self.ttl.clone()}
    }
}

#[derive(Default, Debug)]
pub struct WatchOptions {
    pub(crate) filter: Option<Expression>
}

impl WatchOptions {
    pub fn with_filter(&self, filter: Where) -> Self {
        WatchOptions{filter: Some(filter.into())}
    }
}
