use prost_reflect::ReflectMessage;
use prost_types::Any;
use tracing::error;

static PREFIX: &str = "type.googleapis.com";

pub trait AnyExt {
    fn to_any(&self) -> Any;
    fn from_any(any: &Any) -> Result<Self, prost::DecodeError> where Self: Sized;
}

impl<T> AnyExt for T
where T: ReflectMessage + Default
{
    fn to_any(&self) -> Any {
        Any { type_url: format!("{}/{}", PREFIX, self.descriptor().full_name()), value: self.encode_to_vec() }
    }

    fn from_any(any: &Any) -> Result<Self, prost::DecodeError> {
        let type_url = format!("{}/{}", PREFIX, T::default().descriptor().full_name());
        if type_url != any.type_url {
            error!(type_url = ?any.type_url, expected = ?type_url);
            return Err(prost::DecodeError::new("type_url mismatch"));
        }
        T::decode(any.value.as_ref())
    }
}
